@@ -6,8 +6,8 @@
 #' point cloud with \code{unsmooth_height}.
 #'
 #' This method does not use raster-based methods to smooth the point cloud. This is a true point cloud
-#' smoothing. It is not really useful by itself but may be interesting in combination with filters such
-#' as \link{filter_surfacepoints}, for example to develop new algorithms.
+#' smoothing. It is not really useful by itself but may be interesting in combination with filters,
+#' for example to develop new algorithms.
 #'
 #' @param las An object of class \code{LAS}
 #' @param size numeric. The size of the windows used to smooth.
@@ -23,7 +23,7 @@
 #' LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
 #' las <- readLAS(LASfile, select = "xyz")
 #'
-#' las <- filter_surfacepoints(las, 1)
+#' las <- decimate_points(las, highest(1))
 #' #plot(las)
 #'
 #' las <- smooth_height(las, 5, "gaussian", "circle", sigma = 2)
@@ -46,9 +46,9 @@ smooth_height = function(las, size, method = c("average", "gaussian"), shape = c
 
   force_autoindex(las) <- LIDRGRIDPARTITION
   Zs <- C_smooth(las, size, method, shape, sigma, getThread())
-  fast_quantization(Zs, las@header@PHB[["Z scale factor"]], las@header@PHB[["Z offset"]])
+  fast_quantization(Zs, las[["Z scale factor"]], las[["Z offset"]])
 
-  if (!"Zraw" %in% names(las@data))
+  if (!"Zraw" %in% names(las))
     las@data[["Zraw"]] <- las@data[["Z"]]
 
   las@data[["Z"]] <- Zs
@@ -62,7 +62,7 @@ unsmooth_height = function(las)
   stopifnotlas(las)
   Z <- Zraw <- NULL
 
-  if ("Zraw" %in% names(las@data))
+  if ("Zraw" %in% names(las))
   {
     las@data[["Z"]] <- las@data[["Zraw"]]
     las@data[["Zraw"]] <- NULL
