@@ -115,7 +115,7 @@ raster_value_from_cells = function(raster, cells, layer = 1)
   }
 
   if (is(raster, "SpatRaster"))
-    return(terra::extract(raster, cells, layer = layer)[[1]])
+    return(terra::extract(raster, cells)[[layer]])
 
   raster_error() # nocov
 }
@@ -237,6 +237,13 @@ raster_as_las <- function(raster, bbox = NULL)
 {
   ondisk <- raster_is_proxy(raster)
 
+  # Small rasters can be loaded on the fly
+  if (ondisk & raster_fits_in_memory(raster, n = 10))
+  {
+    raster <- raster_in_memory(raster)
+    ondisk <- FALSE
+  }
+
   if (ondisk & is.null(bbox))
     stop("On-disk rasters not supported without a bbox in 'raster_as_las()'", call. = FALSE) # nocov
 
@@ -406,7 +413,7 @@ raster_materialize <- function(raster, pkg = getOption("lidR.raster.default"), v
 
   if (pkg == "stars")
   {
-    res  <- stars::st_as_stars(bbox, dx = raster$xres, dy = raster$yres, values = values)
+    res  <- stars::st_as_stars(bbox, nx = raster$ncol, ny = raster$nrow, values = values)
     return(res)
   }
 
@@ -665,3 +672,29 @@ raster_class <- function(pkg = getOption("lidR.raster.default"))
   if (pkg == "terra") return("SpatRaster")
   if (pkg == "stars") return("stars")
 }
+
+raster_fits_in_memory <- function(raster, n = 1)
+{
+  if (!raster_is_proxy(raster)) return(TRUE)
+
+  nc <- raster_ncell(raster)
+  n <- n * raster_nlayer(raster)
+  memneed <- nc * n * 8L
+  memavail <- terra::free_RAM()*1000
+  memavail <- 0.6 * memavail
+  return(memneed < memavail)
+}
+
+raster_in_memory <- function(raster)
+{
+  if (is(raster, "stars"))
+    return(stars::st_as_stars(raster))
+
+  if (inherits(raster, "Raster"))
+    return(raster::readAll(raster))
+
+  if (is(raster,"SpatRaster"))
+    return(raster*1)
+
+  raster_error()
+}
